module diCrPKI.d;
/* GENERAL CONSTANTS */
const int PKI_MAX_HASH_LEN  = 40;
const int PKI_DIR_ENCRYPT   =  1;
const int PKI_DIR_DECRYPT   =  0;

/* OPTION FLAGS */
const int PKI_DEFAULT        = 0;
const int PKI_SIG_SHA1RSA    = 0;
const int PKI_SIG_MD5RSA     = 1;
const int PKI_SIG_MD2RSA     = 2;
const int PKI_PBE_SHA_3DES   = 0;
const int PKI_PBE_MD5_DES    = 1;
const int PKI_PBE_MD2_DES    = 2;

const int PKI_HASH_SHA1  = 0;
const int PKI_HASH_MD5   = 1;
const int PKI_HASH_MD2   = 2;
//#define PKI_HASH_DEFINED_

/* nFermatExp values for RSA exponent */
const int PKI_RSAEXP_EQ_3      = 0;
const int PKI_RSAEXP_EQ_5      = 1;
const int PKI_RSAEXP_EQ_17     = 2;
const int PKI_RSAEXP_EQ_257    = 3;
const int PKI_RSAEXP_EQ_65537  = 4;
                                  
const int PKI_KEYGEN_INDICATE  = 0x10;
const int PKI_KEY_NODELAY      = 0x20;
                                     
const int PKI_CMS_FORMAT_BASE64       = 0x10000L;
const int PKI_CMS_EXCLUDE_CERTS       = 0x0100L;
const int PKI_CMS_EXCLUDE_DATA        = 0x0200L;
const int PKI_CMS_INCLUDE_ATTRS       = 0x0800L;
const int PKI_CMS_ADD_SIGNTIME        = 0x1000L;
const int PKI_CMS_ADD_SMIMECAP        = 0x2000L;
//#define PKI_CMS_DEFINED_

const int PKI_X509_FORMAT_PEM         = 0x10000L;
const int PKI_X509_FORMAT_BIN         = 0x20000L;
const int PKI_X509_REQ_KLUDGE         = 0x100000L;
const int PKI_X509_NO_BASIC           = 0x2000000L;				
const int PKI_X509_CA_TRUE            = 0x4000000L;
const int PKI_X509_VERSION1           = 0x8000000L;
/* Flags for Key Usage */
const int PKI_X509_KEYUSAGE_DIGITALSIGNATURE   = 0x0001L;
const int PKI_X509_KEYUSAGE_NONREPUDIATION     = 0x0002L;
const int PKI_X509_KEYUSAGE_KEYENCIPHERMENT    = 0x0004L;
const int PKI_X509_KEYUSAGE_DATAENCIPHERMENT   = 0x0008L;
const int PKI_X509_KEYUSAGE_KEYAGREEMENT       = 0x0010L;
const int PKI_X509_KEYUSAGE_KEYCERTSIGN        = 0x0020L;
const int PKI_X509_KEYUSAGE_CRLSIGN            = 0x0040L;
const int PKI_X509_KEYUSAGE_ENCIPHERONLY       = 0x0080L;
const int PKI_X509_KEYUSAGE_DECIPHERONLY       = 0x0100L;
//#define PKI_X509_DEFINED_


final extern(Windows) export int PKI_Version(int *nMajor, int *nMinor);
final extern(Windows) export int PKI_LastError(char *szErrMsg, int nMsgLen);
final extern(Windows) export int PKI_ErrorCode();
final extern(Windows) export int PKI_CompileTime(char *szTimestamp, int nLen);
final extern(Windows) export int PKI_ModuleName(char *szTimestamp, int nLen, int reserved);
final extern(Windows) export int PKI_PowerUpTests(int nOptions);

/* RFC3369 CRYPTOGRAPHIC MESSAGE SYNTAX FUNCTIONS */
final extern(Windows) export int CMS_MakeEnvData(char *szFileOut, char *szFileIn, char *szCertList, char *sSeed, int nSeedLen, int nOptions);
final extern(Windows) export int CMS_MakeEnvDataFromString(char *szFileOut, char *szDataIn, char *szCertList, char *sSeed, int nSeedLen, int nOptions);
final extern(Windows) export int CMS_ReadEnvData(char *szFileOut, char *szFileIn, char *szX509File, char *szRSAPrivateKey, int nOptions);
final extern(Windows) export int CMS_ReadEnvDataToString(char *szDataOut, int nDataOutLen, char *szFileIn, char *szX509File, char *szRSAPrivateKey, int nOptions);
final extern(Windows) export int CMS_MakeSigData(char *szFileOut, char *szFileIn, char *szCertList, char *szRSAPrivateKey, int nOptions);
final extern(Windows) export int CMS_MakeSigDataFromString(char *szFileOut, char *szDataIn, char *szCertList, char *szRSAPrivateKey, int nOptions);
final extern(Windows) export int CMS_MakeDetachedSig(char *szFileOut, char *szHexDigest, char *szCertList, char *szRSAPrivateKey, int nOptions);
final extern(Windows) export int CMS_ReadSigData(char *szFileOut, char *szFileIn, int nOptions);
final extern(Windows) export int CMS_ReadSigDataToString(char *szDataOut, int nDataOutLen, char *szFileIn, int nOptions);
final extern(Windows) export int CMS_GetSigDataDigest(char *szHexDigestOut, int nDigestLen, char *szFileIn, char *szX509File, int nOptions);

/* RSA KEY FUNCTIONS */
final extern(Windows) export int RSA_MakeKeys(char *szPubKeyFile, char *szPVKFile, int nBits, int nExpFermat, int nTests, int nCount, char *szPassword, char *sSeed, int nSeedLen, int nOptions);
final extern(Windows) export int RSA_ReadEncPrivateKey(char *szOutput, int nOutputLen, char *szPVKFile, char *szPassword, int nOptions);
final extern(Windows) export int RSA_ReadPrivateKeyInfo(char *szOutput, int nOutputLen, char *szKeyFile, int nOptions);
final extern(Windows) export int RSA_GetPrivateKeyFromPFX(char *szOutputFile, char *szPFXFile, int nOptions);
final extern(Windows) export int RSA_ReadPublicKey(char *szOutput, int nOutputLen, char *szKeyFile, int flags);
final extern(Windows) export int RSA_GetPublicKeyFromCert(char *szOutput, int nOutputLen, char *szCertFile, int flags);

final extern(Windows) export int RSA_SavePublicKey(char *szFileOut, char *szKeyString, int nOptions);
final extern(Windows) export int RSA_SavePrivateKeyInfo(char *szFileOut, char *szKeyString, int nOptions);
final extern(Windows) export int RSA_SaveEncPrivateKey(char *szFileOut, char *szKeyString, int nCount, char *szPassword, int nOptions);
final extern(Windows) export int RSA_KeyBits(char *szRsaKey64);
final extern(Windows) export int RSA_KeyBytes(char *szRsaKey64);

/* 'RAW' RSA ENCRYPTION/DECRYPTION FUNCTIONS */
final extern(Windows) export int RSA_RawPublic(ubyte *abData, int nDataLen, char *szPublicKey64, int nOptions);
final extern(Windows) export int RSA_RawPrivate(ubyte *abData, int nDataLen, char *szPrivateKey64, int nOptions);

/* X509 CERTIFICATE FUNCTIONS */
final extern(Windows) export int X509_MakeCert(char *certfile, char *issuerCert, char *subjectPubkeyFile, char *issuerPvkInfoFile, int certnum, int yearsvalid, char *distName, char *email, int keyUsageFlags, char *password, int optionFlags);
final extern(Windows) export int X509_MakeCertSelf(char *certfile, char *epkfile, int certnum, int yearsvalid, char *distName, char *email, int keyUsageFlags, char *password, int optionFlags);
final extern(Windows) export int X509_CertRequest(char *reqfile, char *epkfile, char *distName, char *reserved, char *password, int optionFlags);

final extern(Windows) export int X509_VerifyCert(char *szCertToVerify, char *szIssuerCert, int flags);
final extern(Windows) export int X509_CertThumb(char *szCertFile, char *szHash, int hashlen, int flags);
final extern(Windows) export int X509_CertIsValidNow(char *szCertFile, int flags);
final extern(Windows) export int X509_CertIssuedOn(char *szCertFile, char *szOutput, int nOutputLen, int flags);
final extern(Windows) export int X509_CertExpiresOn(char *szCertFile, char *szOutput, int nOutputLen, int flags);
final extern(Windows) export int X509_CertSerialNumber(char *szCertFile, char *szOutput, int nOutputLen, int flags);
final extern(Windows) export int X509_HashIssuerAndSN(char *szCertFile, char *szOutput, int nOutputLen, int flags);
final extern(Windows) export int X509_CertIssuerName(char *szCertFile, char *szOutput, int nOutputLen, char *szDelim, int flags);
final extern(Windows) export int X509_CertSubjectName(char *szCertFile, char *szOutput, int nOutputLen, char *szDelim, int flags);

/* TRIPLE DES FUNCTIONS */
final extern(Windows) export int TDEA_HexMode(char *output, char *input, char *szHexKey, int bEncrypt, char *szMode, char *sHexIV);
final extern(Windows) export int TDEA_B64Mode(char *output, char *input, char *szB64Key, int bEncrypt, char *szMode, char *sB64IV);
final extern(Windows) export int TDEA_BytesMode(ubyte *output, ubyte *input, int nbytes, ubyte *key, int bEncrypt, char *szMode, ubyte *iv);

/* MISC UTILITIES */
final extern(Windows) export int HASH_HexFromBytes(char *szHexDigest, int digLen, void *aMessage, int messageLen, int flags);
final extern(Windows) export int WIPE_File(char *szFileName, int flags);
final extern(Windows) export int WIPE_Data(void *lpData, int datalen);
final extern(Windows) export int RNG_Bytes(ubyte *output, int out_len, char *seed, int seedlen);
final extern(Windows) export int PWD_Prompt(char *szPassword, int nPwdLen, char *szCaption);
final extern(Windows) export int PWD_PromptEx(char *szPassword, int nPwdLen, char *szCaption, char *szPrompt, int flags);

/* BASE64 AND HEX CONVERSION FUNCTIONS */
final extern(Windows) export int CNV_B64StrFromBytes(char *output, int out_len, ubyte *input, int in_len);
final extern(Windows) export int CNV_BytesFromB64Str(ubyte *output, int out_len, char *input);
final extern(Windows) export int CNV_B64Filter(char *output, char *input, int len);
final extern(Windows) export int CNV_HexStrFromBytes(char *output, int out_len, ubyte *input, int in_len);
final extern(Windows) export int CNV_BytesFromHexStr(ubyte *output, int out_len, char *input);
final extern(Windows) export int CNV_HexFilter(char *output, char *input, int len);
